﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Globalization;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Web.Security;
//using ViewRes;


namespace planerka.DataAccess.Models
{

    #region Models
    [PropertiesMustMatch("NewPassword", "ConfirmPassword", ErrorMessage = "Новый пароль и подтверждение пароля не совпадают.")]
    public class ChangePasswordModel
    {
        [Required(ErrorMessage = "Введите пароль")]
        [DataType(DataType.Password)]
        [DisplayName("Текущий пароль")]
        public string OldPassword { get; set; }

        [Required(ErrorMessage = "Введите новый пароль")]
        [ValidatePasswordLength]
        [DataType(DataType.Password)]
        [DisplayName("Новый пароль")]
        public string NewPassword { get; set; }

        [Required(ErrorMessage = "Введите подтверждение нового пароля")]
        [DataType(DataType.Password)]
        [DisplayName("Подтверждение нового пароля")]
        public string ConfirmPassword { get; set; }
    }

    public class LogOnModel
    {
        [Required(ErrorMessage = "Введите имя пользователя")]
        [DisplayName("Имя пользователя")]
        public string UserName { get; set; }

        [Required(ErrorMessage = "Введите пароль")]
        [DataType(DataType.Password)]
        [DisplayName("Пароль")]
        public string Password { get; set; }

        [DisplayName("Запомнить меня")]
        public bool RememberMe { get; set; }
    }

    //[PropertiesMustMatch("Password", "ConfirmPassword", ErrorMessage = "Пароль и подтверждение пароля не совпадают.")]
    public class RegisterModel
    {
        //[Required(ErrorMessage = "Введите имя пользователя")]
        // имя пользователя - это email
        [DisplayName("Имя пользователя")]
        public string UserName { get; set; }

        [Required(ErrorMessage = "Введите e-mail")]
        [DataType(DataType.EmailAddress)]
        [DisplayName("Email")]
        [RegularExpression(@"\w+([-+.']\w+)*@\w+([-.]\w+)*\.\w+([-.]\w+)*", ErrorMessage = "Неверный e-Mail")]
        public string Email { get; set; }

        [Required(ErrorMessage = "Введите пароль")]
        [ValidatePasswordLength]
        [DataType(DataType.Password)]
        [DisplayName("Пароль")]
        public string Password { get; set; }

        //[Required(ErrorMessage = "Введите подтверждение пароля")]
        //[DataType(DataType.Password)]
        [DisplayName("Подтверждение пароля")]
        public string ConfirmPassword { get; set; }
    }
    #endregion

    #region Services
    // The FormsAuthentication type is sealed and contains static members, so it is difficult to
    // unit test code that calls its members. The interface and helper class below demonstrate
    // how to create an abstract wrapper around such a type in order to make the AccountController
    // code unit testable.

    // При применении к классу модификатор sealed запрещает другим классам наследовать от этого класса. 


    public interface IMembershipService
    {
        int MinPasswordLength { get; }

        bool ValidateUser(string userName, string password);
        MembershipCreateStatus CreateUser(string userName, string password, string email);
        bool ChangePassword(string userName, string oldPassword, string newPassword);
    }

    public class AccountMembershipService : IMembershipService
    {
        private readonly MembershipProvider _provider;

        public AccountMembershipService()
            : this(null)
        {
        }

        public AccountMembershipService(MembershipProvider provider)
        {
            _provider = provider ?? Membership.Provider;
        }

        public int MinPasswordLength
        {
            get
            {
                return _provider.MinRequiredPasswordLength;
            }
        }

        public bool ValidateUser(string userName, string password)
        {
            if (String.IsNullOrEmpty(userName)) throw new ArgumentException("Значение не может быть нулевым или пустым.", "userName");
            if (String.IsNullOrEmpty(password)) throw new ArgumentException("Значение не может быть нулевым или пустым.", "password");

            return _provider.ValidateUser(userName, password);
        }

        public MembershipCreateStatus CreateUser(string userName, string password, string email)
        {
            if (String.IsNullOrEmpty(userName)) throw new ArgumentException("Значение не может быть нулевым или пустым.", "userName");
            if (String.IsNullOrEmpty(password)) throw new ArgumentException("Значение не может быть нулевым или пустым.", "password");
            if (String.IsNullOrEmpty(email)) throw new ArgumentException("Значение не может быть нулевым или пустым.", "email");

            MembershipCreateStatus status;
            _provider.CreateUser(userName, password, email, null, null, true, null, out status);
            return status;
        }

        public bool ChangePassword(string userName, string oldPassword, string newPassword)
        {
            if (String.IsNullOrEmpty(userName)) throw new ArgumentException("Значение не может быть нулевым или пустым.", "userName");
            if (String.IsNullOrEmpty(oldPassword)) throw new ArgumentException("Значение не может быть нулевым или пустым.", "oldPassword");
            if (String.IsNullOrEmpty(newPassword)) throw new ArgumentException("Значение не может быть нулевым или пустым.", "newPassword");

            // The underlying ChangePassword() will throw an exception rather
            // than return false in certain failure scenarios.
            try
            {
                MembershipUser currentUser = _provider.GetUser(userName, true /* userIsOnline */);
                return currentUser.ChangePassword(oldPassword, newPassword);
            }
            catch (ArgumentException)
            {
                return false;
            }
            catch (MembershipPasswordException)
            {
                return false;
            }
        }
    }

    public interface IFormsAuthenticationService
    {
        void SignIn(string userName, bool createPersistentCookie);
        void SignOut();
    }

    public class FormsAuthenticationService : IFormsAuthenticationService
    {
        public void SignIn(string userName, bool createPersistentCookie)
        {
            if (String.IsNullOrEmpty(userName)) throw new ArgumentException("Значение не может быть нулевым или пустым.", "userName");

            FormsAuthentication.SetAuthCookie(userName, createPersistentCookie);
        }

        public void SignOut()
        {
            FormsAuthentication.SignOut();
        }
    }
    #endregion

    #region Validation
    public static class AccountValidation
    {
        public static string ErrorCodeToString(MembershipCreateStatus createStatus)
        {
            // See http://go.microsoft.com/fwlink/?LinkID=177550 for
            // a full list of status codes.
            switch (createStatus)
            {
                case MembershipCreateStatus.DuplicateUserName:
                    return "Пользователь с таким именем существует. Пожалуйста, введите другое имя.";

                case MembershipCreateStatus.DuplicateEmail:
                    return "Пользователь с таким e-mail существует. Пожалуйста, введите другой e-mail.";

                case MembershipCreateStatus.InvalidPassword:
                    return "Пароль неправильный. Пожалуйста, введите правильный пароль.";

                case MembershipCreateStatus.InvalidEmail:
                    return "e-mail неправильный. Пожалуйста, проверьте значение и повторите попытку.";

                case MembershipCreateStatus.InvalidAnswer:
                    return "Ответ на вопрос для восстановления пароля неправильный. Пожалуйста, проверьте значение и повторите попытку.";

                case MembershipCreateStatus.InvalidQuestion:
                    return "Вопрос для восстановления пароля неправильный. Пожалуйста, проверьте значение и повторите попытку.";

                case MembershipCreateStatus.InvalidUserName:
                    return "Неверное имя пользователя. Пожалуйста, проверьте значение и повторите попытку.";

                case MembershipCreateStatus.ProviderError:
                    return "Провайдер аутентификации вернул ошибку. Пожалуйста, проверьте введенные значения и повторите попытку. Если ошибка сохраняется - обратитесь к системному администратору.";

                case MembershipCreateStatus.UserRejected:
                    return "Создание учетной записи пользователя было прервано. Пожалуйста, проверьте введенные значения и повторите попытку.  Если ошибка сохраняется - обратитесь к системному администратору.";

                default:
                    return "Неизвестная ошибка. Пожалуйста, проверьте введенные значения и повторите попытку.  Если ошибка сохраняется - обратитесь к системному администратору.";
            }
        }
    }

    [AttributeUsage(AttributeTargets.Class, AllowMultiple = true, Inherited = true)]
    public sealed class PropertiesMustMatchAttribute : ValidationAttribute
    {
        private const string _defaultErrorMessage = "'{0}' и '{1}' не совпадают.";
        private readonly object _typeId = new object();

        public PropertiesMustMatchAttribute(string originalProperty, string confirmProperty)
            : base(_defaultErrorMessage)
        {
            OriginalProperty = originalProperty;
            ConfirmProperty = confirmProperty;
        }

        public string ConfirmProperty { get; private set; }
        public string OriginalProperty { get; private set; }

        public override object TypeId
        {
            get
            {
                return _typeId;
            }
        }

        public override string FormatErrorMessage(string name)
        {
            return String.Format(CultureInfo.CurrentUICulture, ErrorMessageString,
                OriginalProperty, ConfirmProperty);
        }

        public override bool IsValid(object value)
        {
            PropertyDescriptorCollection properties = TypeDescriptor.GetProperties(value);
            object originalValue = properties.Find(OriginalProperty, true /* ignoreCase */).GetValue(value);
            object confirmValue = properties.Find(ConfirmProperty, true /* ignoreCase */).GetValue(value);
            return Object.Equals(originalValue, confirmValue);
        }
    }

    [AttributeUsage(AttributeTargets.Field | AttributeTargets.Property, AllowMultiple = false, Inherited = true)]
    public sealed class ValidatePasswordLengthAttribute : ValidationAttribute
    {
        private const string _defaultErrorMessage = "'{0}' должен быть минимум {1} символов.";
        private readonly int _minCharacters = Membership.Provider.MinRequiredPasswordLength;

        public ValidatePasswordLengthAttribute()
            : base(_defaultErrorMessage)
        {
        }

        public override string FormatErrorMessage(string name)
        {
            return String.Format(CultureInfo.CurrentUICulture, ErrorMessageString,
                name, _minCharacters);
        }

        public override bool IsValid(object value)
        {
            string valueAsString = value as string;
            return (valueAsString != null && valueAsString.Length >= _minCharacters);
        }
    }
    #endregion

}
